<?php

namespace Moblyze\Asset\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\Asset\Http\Requests\AssetRequest;
use Moblyze\Asset\Interfaces\AssetRepositoryInterface;
use Moblyze\Asset\Models\Asset;

/**
 * Resource controller class for asset.
 */
class AssetResourceController extends BaseController
{

    /**
     * Initialize asset resource controller.
     *
     * @param type AssetRepositoryInterface $asset
     *
     * @return null
     */
    public function __construct(AssetRepositoryInterface $asset)
    {
        parent::__construct();
        $this->repository = $asset;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Asset\Repositories\Criteria\AssetResourceCriteria::class);
    }

    /**
     * Display a list of asset.
     *
     * @return Response
     */
    public function index(AssetRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\Asset\Repositories\Presenter\AssetPresenter::class)
                ->$function();
        }

        $assets = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('asset::asset.names'))
            ->view('asset::asset.index', true)
            ->data(compact('assets', 'view'))
            ->output();
    }

    /**
     * Display asset.
     *
     * @param Request $request
     * @param Model   $asset
     *
     * @return Response
     */
    public function show(AssetRequest $request, Asset $asset)
    {

        if ($asset->exists) {
            $view = 'asset::asset.show';
        } else {
            $view = 'asset::asset.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('asset::asset.name'))
            ->data(compact('asset'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new asset.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AssetRequest $request)
    {

        $asset = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('asset::asset.name')) 
            ->view('asset::asset.create', true) 
            ->data(compact('asset'))
            ->output();
    }

    /**
     * Create new asset.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AssetRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $asset                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('asset::asset.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('asset/asset/' . $asset->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/asset/asset'))
                ->redirect();
        }

    }

    /**
     * Show asset for editing.
     *
     * @param Request $request
     * @param Model   $asset
     *
     * @return Response
     */
    public function edit(AssetRequest $request, Asset $asset)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('asset::asset.name'))
            ->view('asset::asset.edit', true)
            ->data(compact('asset'))
            ->output();
    }

    /**
     * Update the asset.
     *
     * @param Request $request
     * @param Model   $asset
     *
     * @return Response
     */
    public function update(AssetRequest $request, Asset $asset)
    {
        try {
            $attributes = $request->all();

            $asset->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('asset::asset.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('asset/asset/' . $asset->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('asset/asset/' . $asset->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the asset.
     *
     * @param Model   $asset
     *
     * @return Response
     */
    public function destroy(AssetRequest $request, Asset $asset)
    {
        try {

            $asset->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('asset::asset.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('asset/asset/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('asset/asset/' . $asset->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple asset.
     *
     * @param Model   $asset
     *
     * @return Response
     */
    public function delete(AssetRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('asset::asset.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('asset/asset'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/asset/asset'))
                ->redirect();
        }

    }

    /**
     * Restore deleted assets.
     *
     * @param Model   $asset
     *
     * @return Response
     */
    public function restore(AssetRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('asset::asset.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/asset/asset'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/asset/asset/'))
                ->redirect();
        }

    }

}
